@@ -1,8 +1,7 @@
-// Copyright (c) 1997  Per M.A. Bothner.
+// Copyright (c) 1997, 2004  Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.bytecode;
-import java.io.*;
 
 public class Scope
 {
@@ -11,10 +10,24 @@ public class Scope
   Scope nextSibling;
   Scope firstChild, lastChild;
 
-  int start_pc;
-  int end_pc;
+  /** If true, don't call freeLocal on our variables (yet). */
+  boolean preserved;
+
+  Label start;
+  Label end;
   Variable vars;
   Variable last_var;
+
+  public Scope()
+  {
+  }
+
+  public Scope (Label start, Label end)
+  {
+    this.start = start;
+    this.end = end;
+  }
+
   //  Variable lookup (String name);
   public final Variable firstVar () { return vars; }
 
@@ -35,9 +48,7 @@ public class Scope
 
   public Variable addVariable (CodeAttr code, Type type, String name)
   {
-    Variable var = new Variable ();
-    var.setType(type);
-    var.setName(name);
+    Variable var = new Variable(name, type);
     addVariable (code, var);
     return var;
    }
@@ -49,6 +60,7 @@ public class Scope
     else
       last_var.next = var;
     last_var = var;
+    var.scope = this;
   }
 
   /* Add a new Variable, linking it in after a given Variable, */
@@ -68,11 +80,11 @@ public class Scope
       last_var = var;
     if (var.next == var)
       throw new Error("cycle");
+    var.scope = this;
   }
 
   public void addVariable (CodeAttr code, Variable var)
   {
-    var.start_pc = code == null ? 0 : code.PC;
     addVariable (var);
     if (var.isSimple() && code != null)
       var.allocateLocal(code);
@@ -82,7 +94,7 @@ public class Scope
    * Return a variable the scope, by numerical index.
    * @param index the number of the variable
    */
-  Variable find_var (int index) {
+  public Variable getVariable(int index) {
     Variable var = vars;
     while (--index >= 0)
       var = var.next;
@@ -100,11 +112,9 @@ public class Scope
     return true;
   }
 
-  public void setStartPC(int PC)
+  public void setStartPC(CodeAttr code)
   {
-    start_pc = PC;
-    for (Variable var = firstVar ();  var != null;  var = var.nextVar ())
-      var.start_pc = PC;
+    start = code.getLabel();
   }
 
   /**
@@ -112,11 +122,33 @@ public class Scope
    * @param name name to search for
    * @return the Variable, or null if not found (in this scope).
    */
-  Variable lookup (String name) {
+  public Variable lookup (String name) {
     for (Variable var = vars;  var != null;  var = var.next) {
       if (name.equals(var.name))
 	return var;
     }
     return null;
   }
+
+  /** Make local variable slots of this scope availabel for re-use.
+   * However, if the 'preserved' flag is set, defer doing so until
+   * we exit a non-preserved Scope. */
+  void freeLocals (CodeAttr code)
+  {
+    if (preserved)
+      return;
+    for (Variable var = vars; var != null; var = var.next)
+      {
+	if (var.isSimple () && ! var.dead ())
+	  var.freeLocal(code);
+      }
+    for (Scope child = firstChild;  child != null; child = child.nextSibling)
+      {
+	if (child.preserved)
+	  {
+	    child.preserved = false;
+	    child.freeLocals(code);
+	  }
+      }
+  }
 };
