@@ -2,7 +2,6 @@
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.bytecode;
-import java.io.*;
 
 /** The state of a try statement. */
 
@@ -25,17 +24,20 @@ public class TryState {
   /** If the SP > 0 when we entered the try, the stack is saved here. */
   Variable[] savedStack;
 
-  int start_pc;
-  int end_pc;
+  Label start_try;
+  Label end_try;
 
   /** If we are inside a try, the type of variable matched. */
   ClassType try_type;
 
+  /** Only used in emitWithCleanupStart mode. */
+  Type[] savedTypes;
+
   public TryState (CodeAttr code)
   {
     previous = code.try_stack;
     code.try_stack = this;
-    start_pc = code.PC;
+    start_try = code.getLabel();
   }
 
 }
