@@ -8,15 +8,13 @@ package gnu.expr;
 
 public class ChainLambdas extends ExpWalker
 {
-  Compilation comp;
   ScopeExp currentScope;
 
   public static void chainLambdas (Expression exp, Compilation comp)
   {
     ChainLambdas walker = new ChainLambdas();
-    walker.comp = comp;
-    exp.walk(walker);
-    //or:  walter.walkExpression(exp);
+    walker.setContext(comp);
+    walker.walk(exp);
   }
 
   protected Expression walkScopeExp (ScopeExp exp)
@@ -27,6 +25,9 @@ public class ChainLambdas extends ExpWalker
 	exp.outer = currentScope;
 	currentScope = exp;
 	exp.walkChildren(this);
+        exp.setIndexes();
+        if (exp.mustCompile())
+          comp.mustCompileHere();
 	return exp;
       }
     finally
@@ -48,6 +49,7 @@ public class ChainLambdas extends ExpWalker
     try
       {
 	exp.outer = currentScope;
+        exp.firstChild = null;
 	currentScope = exp;
 	exp.walkChildrenOnly(this);
       }
@@ -70,11 +72,14 @@ public class ChainLambdas extends ExpWalker
 
     if (exp.getName() == null && exp.nameDecl != null)
       exp.setName(exp.nameDecl.getName());
+    exp.setIndexes();
+    if (exp.mustCompile())
+      comp.mustCompileHere();
     return exp;
   }
 
   protected Expression walkClassExp (ClassExp exp)
-  {    
+  {
     LambdaExp parent = currentLambda;
     if (parent != null && ! (parent instanceof ClassExp))
       {
@@ -84,14 +89,6 @@ public class ChainLambdas extends ExpWalker
 
     walkScopeExp(exp);
 
-    // Give name to object class.
-    exp.getCompiledClassType(comp);
-    comp.addClass(exp.type);
-    if (exp.isMakingClassPair())
-      {
-	exp.instanceType.setName(exp.type.getName()+"$class");
-	comp.addClass(exp.instanceType);
-      }
     return exp;
   }
 }
