@@ -8,25 +8,20 @@ import gnu.bytecode.*;
 public class ConditionalTarget extends Target
 {
   public Label ifTrue, ifFalse;
-  Interpreter interpreter;
+  Language language;
 
   /**
     * @param ifTrue label to jump to if this evaluates to true
     * @param ifFalse label to jump to if true
+    * @param language specifies what values are true
     */
 
-  public ConditionalTarget (Label ifTrue, Label ifFalse)
-  {
-    this.ifTrue = ifTrue;
-    this.ifFalse = ifFalse;
-  }
-
   public ConditionalTarget (Label ifTrue, Label ifFalse,
-			    Interpreter interpreter)
+			    Language language)
   {
     this.ifTrue = ifTrue;
     this.ifFalse = ifFalse;
-    this.interpreter = interpreter;
+    this.language = language;
   }
 
   /** True if the ifTrue label comes before the ifFalse label.
@@ -64,19 +59,21 @@ public class ConditionalTarget extends Target
 	  }
 	return;
       case 'L':  case '[':
-	comp.compileConstant(interpreter == null ? Boolean.FALSE
-			     : interpreter.booleanObject(false));
+	comp.compileConstant(language == null ? Boolean.FALSE
+			     : language.booleanObject(false));
 	break;
       }
     if (trueBranchComesFirst)
-      {
-	code.emitGotoIfEq(ifFalse);
-	code.emitGoto(ifTrue);
-      }
+      code.emitGotoIfEq(ifFalse);
     else
-      {
-	code.emitGotoIfNE(ifTrue);
-	code.emitGoto(ifFalse);
-      }
+      code.emitGotoIfNE(ifTrue);
+    emitGotoFirstBranch(code);  // Usually a no-op.
+  }
+
+  /** Goto whichever of IfTrue or ifFalse is specified by trueBranchComesFirst.
+   * Normally, the goto should get optimized away as a no-op. */
+  public final void emitGotoFirstBranch(CodeAttr code)
+  {
+    code.emitGoto(trueBranchComesFirst ? ifTrue : ifFalse);
   }
 }
