@@ -3,6 +3,7 @@
 
 package gnu.expr;
 import gnu.bytecode.*;
+import gnu.kawa.reflect.OccurrenceType;
 
 /**
  * A Target which is some variable that implements gnu.lists.Consumer.
@@ -89,14 +90,27 @@ public class ConsumerTarget extends Target
 
 
   public void compileFromStack(Compilation comp, Type stackType)
+  {
+    compileFromStack(comp, stackType, -1);
+  }
+
+  /** Write stack value to Consumer.
+   * @param consumerPushed if -1, then Consumer has not been pushed;
+   *   if 1, Consumer was pushed before value, and value is a known singleton;
+   *   if 0, Consumer was pushed before value, otherwise.
+   */
+  void compileFromStack(Compilation comp,
+                        Type stackType, int consumerPushed)
   {
     CodeAttr code = comp.getCode();
     String methodName = null;
     Method method = null;
     boolean islong = false;
+    stackType = stackType.getImplementationType();
+    char sig;
     if (stackType instanceof PrimType)
       {
-	char sig = stackType.getSignature().charAt(0);
+	sig = stackType.getSignature().charAt(0);
 	switch (sig)
 	  {
 	  case 'B': case 'S': case 'I':
@@ -104,25 +118,30 @@ public class ConsumerTarget extends Target
 	  case 'J':	methodName = "writeLong";  islong = true; break;
 	  case 'F':	methodName = "writeFloat";  break;
 	  case 'D':	methodName = "writeDouble"; islong = true; break;
-	  case 'C':	methodName = "writeChar";  break;
+	  case 'C':	methodName = "append";  break;
 	  case 'Z':	methodName = "writeBoolean";  break;
 	  case 'V':     return;
 	  }
       }
     else
       {
-	if (SeriesTarget.isSingletonType(stackType))
+        sig = '\0';
+	if (consumerPushed == 1 || OccurrenceType.itemCountIsOne(stackType))
 	  methodName = "writeObject";
 	else
 	  {
 	    method = (Compilation.typeValues
 		      .getDeclaredMethod("writeValues", 2));
 	    code.emitLoad(consumer);
+            if (consumerPushed == 0)
+              code.emitSwap();
 	    code.emitInvokeStatic(method);
 	    return;
 	  }
       }
-    if (islong)
+    if (consumerPushed >= 0)
+      ;
+    else if (islong)
       {
 	code.pushScope();
 	Variable temp = code.addLocal(stackType);
@@ -140,7 +159,26 @@ public class ConsumerTarget extends Target
       method = Compilation.typeConsumer.getDeclaredMethod(methodName, 1);
     if (method != null)
       code.emitInvokeInterface(method);
+    if (sig == 'C')
+      code.emitPop(1); // Pop consumer result.
   }
 
+  public boolean compileWrite (Expression exp, Compilation comp)
+  {
+    Type stackType = exp.getType();
+    Type implType = stackType.getImplementationType();
+    if ((implType instanceof PrimType && ! implType.isVoid())
+        || gnu.kawa.reflect.OccurrenceType.itemCountIsOne(implType))
+      {
+        // Optimization to avoid a 'swap'.
+        comp.getCode().emitLoad(this.consumer);
+        Target starget = StackTarget.getInstance(implType);
+        exp.compile(comp, starget);
+        compileFromStack(comp, implType, 1);
+        return true;
+      }
+    return false;
+   }
+
   public Type getType() { return Compilation.scmSequenceType; }
 }
