@@ -1,8 +1,9 @@
-// Copyright (c) 1999  Per M.A. Bothner.
+// Copyright (c) 1999, 2006  Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.expr;
 import gnu.bytecode.*;
+import gnu.mapping.*;
 
 /**
  * Expression to exit a lexically surrounding block.
@@ -26,29 +27,46 @@ public class ExitExp extends Expression
     this.block = block;
   }
 
+  protected boolean mustCompile () { return false; }
+
+  public void apply (CallContext ctx) throws Throwable
+  {
+    throw new BlockExitException(this, result.eval(ctx));
+  }
+
   public void compile (Compilation comp, Target target)
   {
     CodeAttr code = comp.getCode();
     Expression res = result == null ? QuoteExp.voidExp : result;
     res.compileWithPosition(comp, block.subTarget);
+    code.doPendingFinalizers(block.oldTryState);
     code.emitGoto(block.exitLabel);
   }
 
-  Object walk (ExpWalker walker) { return walker.walkExitExp(this); }
-
-  public void print (java.io.PrintWriter ps)
+  protected Expression walk (ExpWalker walker)
   {
-    ps.print("(#%exit ");
+    return walker.walkExitExp(this);
+  }
+
+  protected void walkChildren (ExpWalker walker)
+  {
+    result = walker.walk(result);
+  }
+
+  public void print (OutPort out)
+  {
+    out.startLogicalBlock("(Exit", false, ")");
+    out.writeSpaceFill();
     if (block == null || block.label == null)
-      ps.print("<unknown>");
+      out.print("<unknown>");
     else
-      ps.print(block.label.getName());
+      out.print(block.label.getName());
     if (result != null)
       {
-	ps.print(' ');
-	result.print(ps);
+	out.writeSpaceLinear();
+	result.print(out);
       }
-    ps.print(')');
+    out.endLogicalBlock(")");
   }
 
   public Type getType()
