@@ -1,21 +1,22 @@
 package gnu.expr;
+import gnu.bytecode.Type;
 
 /** Does setTailCall on ApplyExp's that are tail-calls.
     Also setCanRead, setCanCall, setCanWrite on Declarations
     and setCanRead, setCanCall on LambdaExp when appropriate. */
 
-public class FindTailCalls extends ExpFullWalker
+public class FindTailCalls extends ExpWalker
 {
-  public static void findTailCalls (Expression exp)
+  public static void findTailCalls (Expression exp, Compilation comp)
   {
     FindTailCalls walker = new FindTailCalls();
-    exp.walk(walker);
-    //or:  walter.walkExpression(exp);
+    walker.setContext(comp);
+    walker.walk(exp);
   }
 
   boolean inTailContext = true;
 
-  public Object walkApplyExp(ApplyExp exp)
+  protected Expression walkApplyExp(ApplyExp exp)
   {
     if (inTailContext)
       exp.setTailCall(true);
@@ -25,26 +26,42 @@ public class FindTailCalls extends ExpFullWalker
     try
       {
 	inTailContext = false;
+	boolean isAppendValues = false;
 	if (exp.func instanceof ReferenceExp)
 	  {
 	    ReferenceExp func = (ReferenceExp) exp.func;
-	    Declaration binding = func.binding;
+	    Declaration binding = Declaration.followAliases(func.binding);
 	    if (binding != null)
 	      {
-		exp.nextCall = binding.firstCall;
-		binding.firstCall = exp;
-		binding.setCanCall(true);
+                // No point in building chain if STATIC_SPECIFIED, and it can
+                // lead to memory leaks.  At least if interactive calls cam
+                // resolve to previously-compiled Declarations (as in XQuery).
+                if (! binding.getFlag(Declaration.STATIC_SPECIFIED))
+                  {
+                    exp.nextCall = binding.firstCall;
+                    binding.firstCall = exp;
+                  }
+                Compilation comp = getCompilation();
+		binding.setCanCall();
+                if (! comp.mustCompile)
+                  // Avoid tricky optimization if we're interpreting.
+                  binding.setCanRead();
 		Expression value = binding.getValue();
 		if (value instanceof LambdaExp)
 		  lexp = (LambdaExp) value;
 	      }
 	  }
-	else if (exp.func instanceof LambdaExp)
+	else if (exp.func instanceof LambdaExp
+		 && ! (exp.func instanceof ClassExp))
 	  {
 	    lexp = (LambdaExp) exp.func;
 	    walkLambdaExp(lexp, false);
 	    lexp.setCanCall(true);
 	  }
+	else if (exp.func instanceof QuoteExp
+		 && (((QuoteExp) exp.func).getValue()
+		     == gnu.kawa.functions.AppendValues.appendValues))
+	  isAppendValues = true;
 	else
 	  exp.func = (Expression) exp.func.walk(this);
 	if (lexp != null)
@@ -57,7 +74,15 @@ public class FindTailCalls extends ExpFullWalker
 	    else
 	      lexp.returnContinuation = LambdaExp.unknownContinuation;
 	  }
-	exp.args = walkExps(exp.args);
+	if (isAppendValues && exp.args.length > 0)
+	  {
+	    int last = exp.args.length - 1;
+	    exp.args = walkExps(exp.args, last);
+	    inTailContext = save;
+	    exp.args[last] = walk(exp.args[last]);
+	  }
+	else
+	  exp.args = walkExps(exp.args);
 	return exp;
       }
     finally
@@ -66,12 +91,12 @@ public class FindTailCalls extends ExpFullWalker
       }
   }
 
-  public Object walkBeginExp(BeginExp exp)
+  protected Expression walkBeginExp(BeginExp exp)
   {
     boolean save = inTailContext;
     try
       {
-	int n = exp.exps.length - 1;
+	int n = exp.length - 1;
 	for (int i = 0;  i <= n;  i++)
 	  {
 	    inTailContext = (i == n) && save;
@@ -85,18 +110,32 @@ public class FindTailCalls extends ExpFullWalker
       }
   }
 
-  public Object walkFluidLetExp (FluidLetExp exp)
+  protected Expression walkFluidLetExp (FluidLetExp exp)
   {
     for (Declaration decl = exp.firstDecl();
          decl != null; decl = decl.nextDecl())
       {
         decl.setCanRead(true);
         decl.setCanWrite(true);
+        if (decl.base != null)
+          {
+            decl.base.setCanRead(true);
+            decl.base.setCanWrite(true);
+          }
+      }
+    boolean save = inTailContext;
+    inTailContext = false;
+    try
+      {
+	return super.walkFluidLetExp(exp);
+      }
+    finally
+      {
+	inTailContext = save;
       }
-    return super.walkFluidLetExp(exp);
   }
 
-  public Object walkLetExp (LetExp exp)
+  protected Expression walkLetExp (LetExp exp)
   {
     int n = exp.inits.length; 
     boolean save = inTailContext;
@@ -107,19 +146,35 @@ public class FindTailCalls extends ExpFullWalker
 
 	decl = exp.firstDecl();
 	for (int i = 0;  i < n;  i++, decl = decl.nextDecl())
-	  exp.inits[i] = walkSetExp (decl, exp.inits[i]);
+	  {
+	    Expression init = walkSetExp (decl, exp.inits[i]);
+	    // Optimize letrec-like forms.
+	    if (init == QuoteExp.undefined_exp)
+	      {
+		Expression value = decl.getValue();
+		if (value instanceof LambdaExp
+		    || (value != init && value instanceof QuoteExp))
+		  init = value;
+	      }
+	    exp.inits[i] = init;
+	  }
       }
     finally
       {
 	inTailContext = save;
       }
     exp.body = (Expression) exp.body.walk(this);
+    walkDecls(exp);
+    return exp;
+  }
 
-    decl = exp.firstDecl();
-    for (int i = 0;  i < n;  i++, decl = decl.nextDecl())
+  public void walkDecls (ScopeExp exp)
+  {
+    Declaration decl = exp.firstDecl();
+    for (;  decl != null;  decl = decl.nextDecl())
       {
 	Expression value = decl.getValue();
-	if (value != null && value instanceof LambdaExp)
+	if (value instanceof LambdaExp)
 	  {
 	    LambdaExp lexp = (LambdaExp) value;
 	    if (decl.getCanRead())
@@ -127,11 +182,18 @@ public class FindTailCalls extends ExpFullWalker
 	    if (decl.getCanCall())
 	      lexp.setCanCall(true);
 	  }
+        if (decl.getFlag(Declaration.EXPORT_SPECIFIED)
+            && value instanceof ReferenceExp)
+          {
+            ReferenceExp rexp = (ReferenceExp) value;
+            Declaration context = rexp.contextDecl();
+            if (context != null && context.isPrivate())
+              context.setFlag(Declaration.EXTERNAL_ACCESS);
+          }
       }
-    return exp;
   }
 
-  public Object walkIfExp (IfExp exp)
+  protected Expression walkIfExp (IfExp exp)
   {
     boolean save = inTailContext;
     try
@@ -150,7 +212,7 @@ public class FindTailCalls extends ExpFullWalker
     return exp;
   }
 
-  public Object walkLambdaExp (LambdaExp exp)
+  protected Expression walkLambdaExp (LambdaExp exp)
   {
     walkLambdaExp (exp, true);
     return exp;
@@ -168,7 +230,7 @@ public class FindTailCalls extends ExpFullWalker
 	inTailContext = false;
 	if (exp.defaultArgs != null)
 	  exp.defaultArgs = walkExps(exp.defaultArgs);
-	inTailContext = true;
+	inTailContext = exp.getInlineOnly() ? save : true;
 	if (exitValue == null && exp.body != null)
 	  exp.body = (Expression) exp.body.walk(this);
       }
@@ -178,17 +240,20 @@ public class FindTailCalls extends ExpFullWalker
 	currentLambda = parent;
       }
 
+    walkDecls(exp);
+
     for (LambdaExp child = exp.firstChild;  child != null;
 	 child = child.nextSibling)
       {
 	if (child.getCanRead()
+	    || child.isClassMethod()
 	    || child.min_args != child.max_args)
 	  child.flags |= LambdaExp.CANNOT_INLINE;
 	else
 	  {
 	    ApplyExp caller = child.returnContinuation;
 	    if (caller != LambdaExp.unknownContinuation
-		&& ! Compilation.usingCPStyle())
+		&& ! comp.usingCPStyle())
 	      {
 		child.setInlineOnly(true);
 	      }
@@ -202,7 +267,7 @@ public class FindTailCalls extends ExpFullWalker
 	  continue;
 	// We can inline child if it is a member of a set of functions
 	// which can all be inlined in the same method, and for which
-	// all callers are known and members of the same,
+	// all callers are known and members of the same set,
 	// and each function has at most one caller that is not a tail-call.
 	// FIXME  Basic algorithm:
 	/*
@@ -233,7 +298,7 @@ public class FindTailCalls extends ExpFullWalker
   // calls that call the key.
   // Hashtable applications = new Hashtable();
 
-  public Object walkObjectExp (ObjectExp exp)
+  protected Expression walkClassExp (ClassExp exp)
   {
     boolean save = inTailContext;
     LambdaExp parent = currentLambda;
@@ -255,19 +320,29 @@ public class FindTailCalls extends ExpFullWalker
     return exp;
   }
 
-  public Object walkReferenceExp (ReferenceExp exp)
+  protected Expression walkReferenceExp (ReferenceExp exp)
   {
-    if (exp.binding != null)
-      exp.binding.setCanRead(true);
+    Declaration decl = Declaration.followAliases(exp.binding);
+    if (decl != null)
+      {
+        // Replace references to a void variable (including one whose value
+        // is the empty sequence in XQuery) by an empty constant.  This is
+        // not so much an optimization as avoiding the complications and
+        // paradoxes of variables and expression that are void.
+        if (decl.type == Type.void_type)
+          return QuoteExp.voidExp;
+        decl.setCanRead(true);
+      }
+    Declaration ctx = exp.contextDecl();
+    if (ctx != null)
+      ctx.setCanRead(true);
     return exp;
   }
 
   final Expression walkSetExp (Declaration decl, Expression value)
   {
-    if (decl != null)
-      decl.setCanWrite(true);
-    if (decl != null && decl.getValue() == value && value instanceof LambdaExp
-	&& ! (value instanceof ObjectExp)
+    if (decl != null && decl.getValue() == value
+	&& value instanceof LambdaExp && ! (value instanceof ClassExp)
         && ! decl.isPublic())
       {
 	LambdaExp lexp = (LambdaExp) value; 
@@ -278,13 +353,37 @@ public class FindTailCalls extends ExpFullWalker
       return (Expression) value.walk(this);
   }
 
-  public Object walkSetExp (SetExp exp)
+  protected Expression walkSetExp (SetExp exp)
   {
     boolean save = inTailContext;
     try
       {
 	inTailContext = false;
-	exp.new_value = walkSetExp(exp.binding, exp.new_value);
+	Declaration decl = exp.binding;
+	if (decl != null && decl.isAlias())
+	  {
+	    if (exp.isDefining())
+	      {
+		exp.new_value = (Expression) exp.new_value.walk(this);
+		return exp;
+	      }
+	    decl = Declaration.followAliases(decl);
+	  }
+	if (decl != null)
+	  decl.setCanWrite();
+        Declaration ctx = exp.contextDecl();
+        if (ctx != null)
+          ctx.setCanRead(true);
+	Expression value = walkSetExp(decl, exp.new_value);
+	if (decl != null && decl.context instanceof LetExp
+	    && value == decl.getValue()
+	    && (value instanceof LambdaExp || value instanceof QuoteExp))
+	  {
+	    // The assignment is redundant, as it has been moved to the
+	    // initialization of the LetExp.
+	    return QuoteExp.voidExp;
+	  }
+	exp.new_value = value;
 	return exp;
       }
     finally
@@ -293,7 +392,7 @@ public class FindTailCalls extends ExpFullWalker
       }
   }
 
-  public Object walkTryExp (TryExp exp)
+  protected Expression walkTryExp (TryExp exp)
   {
     boolean save = inTailContext;
     try
@@ -307,7 +406,7 @@ public class FindTailCalls extends ExpFullWalker
       }
   }
 
-  public Object walkSynchronizedExp (SynchronizedExp exp)
+  protected Expression walkSynchronizedExp (SynchronizedExp exp)
   {
     boolean save = inTailContext;
     try
