@@ -224,7 +224,9 @@ public class ModuleInfo
 
   public static void register (Object instance)
   {
-    ModuleInfo info = find(instance.getClass().getName());
+    Class clas = instance.getClass();
+    ModuleInfo info = find(clas.getName());
+    info.moduleClass = clas;
     ModuleContext.getContext().setInstance(info, instance);
   }
 
@@ -353,12 +355,7 @@ public class ModuleInfo
       {
         try
           {
-            /* #ifdef JAVA2 */
-            moduleClass = Class.forName(className, false,
-                                        manager.defaultClassLoader);
-            /* #else */
-            // moduleClass = Class.forName(className);
-            /* #endif */
+            moduleClass = ClassType.getContextClass(className);
           }
         catch (ClassNotFoundException ex)
           {
