@@ -1,14 +1,11 @@
 package gnu.expr;
 import gnu.bytecode.*;
-import gnu.mapping.*;
 
 public class ObjectExp extends ClassExp
 {
   public ObjectExp ()
   {
-    type = null;
-    // Make sure we actually generate a class.
-    setCanRead(true);
+    super(true);
   }
 
   protected Expression walk (ExpWalker walker)
@@ -18,11 +15,11 @@ public class ObjectExp extends ClassExp
 
   public void compile (Compilation comp, Target target)
   {
-    ClassType new_class = compile (comp);
+    compile (comp);
     CodeAttr code = comp.getCode();
     code.emitNew(type);
     code.emitDup(1);
-    Method init = comp.getConstructor(type, this);
+    Method init = Compilation.getConstructor(type, this);
     if (closureEnvField != null)
       {
 	LambdaExp caller = outerLambda();
