@@ -1,13 +1,15 @@
 package gnu.jemacs.lang;
 import gnu.expr.*;
 import kawa.lang.*;
-import gnu.kawa.util.*;
+import gnu.lists.*;
 import gnu.mapping.*;
 
 /** Implement the ELisp `while' syntax form. */
 
-public class While extends Syntax implements Printable
+public class While extends Syntax
 {
+  static kawa.standard.begin begin = new kawa.standard.begin();
+
   public Expression rewriteForm (Pair form, Translator tr)
   {
     Object obj = form.cdr;
@@ -15,7 +17,6 @@ public class While extends Syntax implements Printable
       return tr.syntaxError("missing arguments for while");
     tr.mustCompileHere();
     form = (Pair) obj;
-    return Expression.makeWhile(tr.rewrite(form.car),
-				tr.rewrite_body(form.cdr));
+    return Expression.makeWhile(form.car, new Pair (begin, form.cdr), tr);
   }
 }
