@@ -4,10 +4,12 @@
 package gnu.kawa.xml;
 import gnu.bytecode.*;
 import gnu.mapping.Procedure;
+import gnu.mapping.Values;
 import gnu.expr.*;
 import gnu.text.Printable;
 import gnu.math.*;
 import java.math.BigDecimal;
+import gnu.lists.SeqPosition;
 import gnu.lists.Consumer;
 import gnu.xml.TextUtils;
 import gnu.text.URIPath;
@@ -28,6 +30,7 @@ public class XDataType extends Type implements TypeValue
   /** One of the {@code XXXX_TYPE_CODE} constants. */
   int typeCode;
 
+  public static final int ANY_ATOMIC_TYPE_CODE = 1;
   public static final int DECIMAL_TYPE_CODE = 2;
   public static final int INTEGER_TYPE_CODE = 3;
   public static final int NON_POSITIVE_INTEGER_TYPE_CODE = 4;
@@ -70,6 +73,16 @@ public class XDataType extends Type implements TypeValue
 
   public static final int STRING_TYPE_CODE = 36;
 
+  public static final int NORMALIZED_STRING_TYPE_CODE = 37;
+  public static final int TOKEN_TYPE_CODE = 38;
+  public static final int LANGUAGE_TYPE_CODE = 39;
+  public static final int NMTOKEN_TYPE_CODE = 40;
+  public static final int NAME_TYPE_CODE = 41;
+  public static final int NCNAME_TYPE_CODE = 42;
+  public static final int ID_TYPE_CODE = 43;
+  public static final int IDREF_TYPE_CODE = 44;
+  public static final int ENTITY_TYPE_CODE = 45;
+
   public XDataType (Object name, Type implementationType, int typeCode)
   {
     super(implementationType);
@@ -80,8 +93,26 @@ public class XDataType extends Type implements TypeValue
     this.typeCode = typeCode;
   }
 
+  public static final XDataType anyAtomicType =
+    new XDataType("anyAtomicType",
+                  Type.pointer_type,
+                  ANY_ATOMIC_TYPE_CODE);
+
   public static final XDataType stringType =
-    new XDataType("string", ClassType.make("java.lang.String"),
+    new XDataType("string",
+                  /* #ifdef use:java.lang.CharSequence */
+                  ClassType.make("java.lang.CharSequence"),
+                  /* #else */
+                  // ClassType.make("java.lang.String"),
+                  /* #endif */
+                  STRING_TYPE_CODE);
+
+  /** A value implemented as java.lang.String.
+   * Can be cast from CharSequence.
+   */
+  public static final XDataType stringStringType =
+    new XDataType("String",
+                  ClassType.make("java.lang.String"),
                   STRING_TYPE_CODE);
 
   public static final XDataType untypedAtomicType =
@@ -201,8 +232,14 @@ public class XDataType extends Type implements TypeValue
   {
     switch (typeCode)
       {
+      case ANY_ATOMIC_TYPE_CODE:
+        return ! (obj instanceof Values || obj instanceof SeqPosition);
       case STRING_TYPE_CODE:
-        return obj instanceof java.lang.String;
+        /* #ifdef use:java.lang.CharSequence */
+        return obj instanceof java.lang.CharSequence;
+        /* #else */
+        // return obj instanceof java.lang.String;
+        /* #endif */
       case UNTYPED_ATOMIC_TYPE_CODE:
         return obj instanceof gnu.kawa.xml.UntypedAtomic;
       case ANY_URI_TYPE_CODE:
@@ -401,7 +438,9 @@ public class XDataType extends Type implements TypeValue
 
   public int compare(Type other)
   {
-    if (this == other)
+    if (this == other
+        || (this == stringStringType && other == stringType)
+        || (this == stringType && other == stringStringType))
       return 0;
     return implementationType.compare(other); // FIXME
   }
