@@ -1,4 +1,4 @@
-// Copyright (c) 2001  Per M.A. Bothner and Brainfood Inc.
+// Copyright (c) 2001, 2002  Per M.A. Bothner and Brainfood Inc.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.lists;
@@ -6,7 +6,11 @@ import java.io.*;
 
 /** Simple adjustable-length vector of unsigned 16-bit integers (shorts). */
 
-public class U16Vector extends SimpleVector implements Externalizable
+public class U16Vector extends SimpleVector
+  implements Externalizable
+  /* #ifdef JAVA2 */
+  , Comparable
+  /* #endif */
 {
   short[] data;
 
@@ -123,7 +127,7 @@ public class U16Vector extends SimpleVector implements Externalizable
 
   public String getTag() { return "u16"; }
 
-  public boolean consumeNext(int ipos, Object xpos, Consumer out)
+  public boolean consumeNext (int ipos, Consumer out)
   {
     int index = ipos >>> 1;
     if (index >= size)
@@ -132,17 +136,23 @@ public class U16Vector extends SimpleVector implements Externalizable
     return true;
   }
 
-  public void consume(int iposStart, Object xposStart,
-		      int iposEnd, Object xposEnd, Consumer out)
+  public void consumePosRange (int iposStart, int iposEnd, Consumer out)
   {
     if (out.ignoring())
       return;
     int i = iposStart >>> 1;
     int end = iposEnd >>> 1;
+    if (end > size)
+      end = size;
     for (;  i < end;  i++)
       out.writeInt(data[i] & 0xffff);
   }
 
+  public int compareTo(Object obj)
+  {
+    return compareToInt(this, (U16Vector) obj);
+  }
+
   /**
    * @serialData Write 'size' (using writeInt),
    *   followed by 'size' elements in order (using writeShort).
