@@ -35,6 +35,17 @@ public class LiteralFormat extends ReportFormat
     throw new Error("LiteralFormat.parseObject - not implemented");
   }
 
+  /** Return the text that would be printed by the format. */
+  public String content ()
+  {
+    return new String(text);
+  }
+
   public String toString()
-  { return "LiteralFormat[\"" + text + "\"]"; }
+  {
+    StringBuffer sbuf = new StringBuffer("LiteralFormat[\"");
+    sbuf.append(text);
+    sbuf.append("\"]");
+    return sbuf.toString();
+  }
 }
