@@ -14,12 +14,22 @@ import gnu.math.DFloNum;
 
 public class SequenceUtils
 {
-  public static Object zeroOrOne (Object arg)
+  public static boolean isZeroOrOne (Object arg)
   {
     // Assumes arg is normalized.
-    if (arg instanceof Values && ! ((Values) arg).isEmpty())
-      throw new IllegalArgumentException();
-    return arg;
+    return ! (arg instanceof Values) || ((Values) arg).isEmpty();
+  }
+
+  static Object coerceToZeroOrOne (Object arg, String functionName, int iarg)
+  {
+    if (isZeroOrOne(arg))
+      return arg;
+    throw new WrongType(functionName, iarg, arg, "xs:item()?");
+  }
+
+  public static Object zeroOrOne (Object arg)
+  {
+    return coerceToZeroOrOne(arg, "zero-or-one", 1);
   }
 
   public static Object oneOrMore (Object arg)
@@ -180,7 +190,7 @@ public class SequenceUtils
                                    NamedCollator collator)
   {
     int kind1 = seq1.getNextKind(ipos1);
-    int kind2 = seq1.getNextKind(ipos2);
+    int kind2 = seq2.getNextKind(ipos2);
     switch (kind1)
       {
       case Sequence.ELEMENT_VALUE:
