@@ -2,34 +2,28 @@ package kawa.lang;
 import gnu.mapping.*;
 
 /**
- * A Continuation "rerpesents an entire (default) future for the computation.
+ * A Continuation "represents an entire (default) future for the computation.
  * This implemementation is based on Java exceptions, and is restricted
  * to "upward" (?) continuation (i.e. catch/throw-type uses).
  * @author	Per Bothner
  */
 
-public class Continuation extends ProcedureN
+public class Continuation extends MethodProc
 {
   public boolean invoked;
   static int counter;
   int id;
 
-  public Continuation ()
+  public Continuation (CallContext ctx)
   {
-    id = ++counter;
   }
 
-  public Object apply1 (Object arg1)
+  public void apply (CallContext ctx)
   {
     if (invoked)
       throw new GenericError
 	("implementation restriction: continuation can only be used once");
-    throw new CalledContinuation (arg1, this);
-  }
-
-  public Object applyN (Object[] args)
-  {
-    return apply1 (Values.make (args));
+    throw new CalledContinuation (ctx.values, this);
   }
 
   public final String toString()
