@@ -1,5 +1,7 @@
 package kawa.lang;
 import gnu.mapping.*;
+import gnu.text.Printable;
+import gnu.lists.Consumer;
 
 /** Implement Scheme "promises".
  * @author Per Bothner
@@ -29,15 +31,15 @@ public class Promise implements Printable
     return result;
   }
 
-  public void print(java.io.PrintWriter ps)
+  public void print (Consumer out)
   {
-    ps.print ("#<promise ");
     if (result == null)
-      ps.print (" - not forced yet>");
+      out.write("#<promise - not forced yet>");
     else
       {
-	SFormat.print (result, ps);
-	ps.print ('>');
+	out.write("#<promise - forced to a ");
+	out.write(result.getClass().getName());
+	out.write ('>');
       }
   }
 }
