@@ -2,6 +2,7 @@ package kawa.lang;
 import gnu.mapping.*;
 import gnu.expr.*;
 import gnu.lists.*;
+import gnu.text.Printable;
 
 /**
  * Abstract class for "syntax" objects.
@@ -63,7 +64,7 @@ abstract public class Syntax implements Printable, Named
   {
     boolean ok = scanForDefinitions(st, tr.formStack, defs, tr);
     if (! ok)
-      tr.formStack.add(new ErrorExp("syntax error"));
+      tr.formStack.add(new ErrorExp("syntax error expanding "+this));
   }
 
   /** Check if a statement is a definition, for initial pass.
@@ -81,14 +82,11 @@ abstract public class Syntax implements Printable, Named
     return true;
   }
 
-  public void print(java.io.PrintWriter ps)
+  public void print (Consumer out)
   {
-    ps.print("#<syntax ");
+    out.write("#<syntax ");
     String name = this.getName();
-    if (name == null)
-      ps.print ("<unnamed>");
-    else
-      ps.print(name);
-    ps.print ('>');
+    out.write(name == null ? "<unnamed>" : name);
+    out.write('>');
   }
 }
