@@ -78,9 +78,6 @@ public class SyntaxTemplate implements Externalizable
 
   static final String dots3 = "...";
 
-  /** The lexical context of template definition. */
-  private ScopeExp currentScope;
-
   /* DEBUGGING:
   void print_template_program (java.util.Vector patternNames,
 			       java.io.PrintWriter ps)
@@ -164,7 +161,6 @@ public class SyntaxTemplate implements Externalizable
 
   public SyntaxTemplate (Object template, SyntaxForm syntax, Translator tr)
   {
-    this.currentScope = tr.currentScope();
     this.patternNesting = tr == null || tr.patternScope == null ? ""
       : tr.patternScope.patternNesting.toString();
     StringBuffer program = new StringBuffer ();
@@ -189,8 +185,11 @@ public class SyntaxTemplate implements Externalizable
 	err.print(" for ");
 	err.print(macro);
       }
-    err.println(':');
-    print_template_program (tr.patternScope.pattern_names, err);
+    if (tr != null && tr.patternScope != null)
+      {
+        err.println(" - ");
+        print_template_program (tr.patternScope.pattern_names, err);
+      }
     err.println ('}');
     */
   }
@@ -224,14 +223,14 @@ public class SyntaxTemplate implements Externalizable
     /* #ifdef use:java.util.IdentityHashMap */ 
     if (form instanceof Pair || form instanceof FVector)
       {
-	IdentityHashMap seen_map = (IdentityHashMap) seen;
-	if (seen_map.containsKey(form))
-	  {
-	    /* FIXME cycles are OK if data are literal. */
-	    tr.syntaxError("self-referential (cyclic) syntax tenplate");
-	    return -2;
-	  }
-	seen_map.put(form, form);
+        IdentityHashMap seen_map = (IdentityHashMap) seen;
+        if (seen_map.containsKey(form))
+          {
+            /* FIXME cycles are OK if data are literal. */
+            tr.syntaxError("self-referential (cyclic) syntax template");
+            return -2;
+          }
+        seen_map.put(form, form);
       }
     /* #endif */
 
@@ -246,7 +245,7 @@ public class SyntaxTemplate implements Externalizable
         // Look for (... ...) and translate that to ...
         if (tr.matches(car, dots3))
           {
-            Object cdr = tr.stripSyntax(pair.cdr);
+            Object cdr = Translator.stripSyntax(pair.cdr);
             if (cdr instanceof Pair)
               {
                 Pair cdr_pair = (Pair) cdr;
@@ -384,24 +383,31 @@ public class SyntaxTemplate implements Externalizable
    * The compiler translates <code>(syntax <var>template</var>)</code>
    * to a call to this method.
    */
-  public Object execute (Object[] vars)
+  public Object execute (Object[] vars, TemplateScope templateScope)
   {
     if (false)  // DEBUGGING
       {
 	OutPort err = OutPort.errDefault();
 	err.print("{Expand template in ");
 	err.print(((Translator) Compilation.getCurrent()).getCurrentSyntax());
-	err.print(" vars: ");
-	for (int i = 0;  i < vars.length;  i++)
-	  {
-	    err.println();
-	    err.print("  " + i +" : ");
-	    kawa.standard.Scheme.writeFormat.writeObject(vars[i], err);
-	  }
+        err.print(" tscope: ");
+        err.print(templateScope);
+        if (false && vars != null)
+          {
+            err.print(" vars: ");
+            for (int i = 0;  i < vars.length;  i++)
+              {
+                err.println();
+                err.print("  " + i +" : ");
+                kawa.standard.Scheme.writeFormat.writeObject(vars[i], err);
+              }
+          }
 	err.println('}');
       }
 
-    Object result = execute(vars, (Translator) Compilation.getCurrent());
+    Object result = execute(0, vars, 0, new int[max_nesting],
+                            (Translator) Compilation.getCurrent(),
+                            templateScope);
 
     if (false) // DEBUGGING:
       {
@@ -419,20 +425,9 @@ public class SyntaxTemplate implements Externalizable
     return result;
   }
 
-  public Object execute (Object[] vars, Translator tr)
+  public Object execute (Object[] vars, Translator tr,
+                         TemplateScope templateScope)
   {
-    TemplateScope templateScope = new TemplateScope();
-    if (currentScope != null)
-      templateScope.outer = currentScope;
-    else
-      {
-	Syntax curSyntax = tr.getCurrentSyntax();
-	if (curSyntax instanceof Macro)
-	  {
-	    templateScope.outer = ((Macro) curSyntax).getCapturedScope();
-	    templateScope.macroContext = tr.macroContext;
-	  }
-      }
     return execute(0, vars, 0, new int[max_nesting], tr, templateScope);
   }
 
@@ -527,11 +522,9 @@ public class SyntaxTemplate implements Externalizable
       {
 	Pair p = null;
 	Object result = null;
-	int pc0=pc, pc1=pc;
 	for (;;)
 	  {
 	    pc++;
-	    pc1=pc;
 	    Object q
 	      = executeToList(pc, vars, nesting, indexes, tr, templateScope);
 	    if (p == null)
