@@ -1,10 +1,13 @@
 package kawa.standard;
-import kawa.lang.*;
 import gnu.mapping.*;
 
 public class call_with_values extends Procedure2
 {
+  public static final call_with_values callWithValues = new call_with_values();
+  static { callWithValues.setName("call-with-values"); }
+
   public static Object callWithValues (Procedure producer, Procedure consumer)
+     throws Throwable
   {
     Object values = producer.apply0();
     if (values instanceof Values)
@@ -13,26 +16,26 @@ public class call_with_values extends Procedure2
       return consumer.apply1(values);
   }
 
-  public Object apply2 (Object producer, Object consumer)
+  public Object apply2 (Object producer, Object consumer) throws Throwable
   {
     return callWithValues((Procedure) producer, (Procedure) consumer);
   }
 
-  public void apply (CallStack stack)
+  public void apply (CallContext ctx) throws Throwable
   {
     Procedure.checkArgCount(this, 2);
-    Object[] args = stack.args;
+    Object[] args = ctx.getArgs();
     Object values = ((Procedure) args[0]).apply0 ();
     Procedure consumer = (Procedure) args[1];
     if (values instanceof Values)
-      args = ((Values) values).getValues();
+      {
+	args = ((Values) values).getValues();
+	consumer.checkN(args, ctx);
+      }
     else
       {
-	args = new Object[1];
-	args[0] = values;
+	consumer.check1(values, ctx);
       }
-    stack.args = args;
-    consumer.apply(stack);
   }
 }
 
