@@ -6,7 +6,7 @@ import gnu.expr.*;
 import gnu.text.SourceMessages;
 import gnu.text.SyntaxException;
 import gnu.lists.*;
-import gnu.text.URI_utils;
+import gnu.text.*;
 import java.net.URL;
 
 public class load extends Procedure1 {
@@ -51,15 +51,18 @@ public class load extends Procedure1 {
       }
   }
 
-  public final static void loadCompiled (String name, Environment env)
+  public final static void loadCompiled (Path path, Environment env)
     throws Throwable
   {
     Environment orig_env = Environment.getCurrent();
+    String name = path.toString();
     try
       {
 	if (env != orig_env)
 	  Environment.setCurrent(env);
-	File zfile = new File (name);
+        if (! (path instanceof FilePath))
+          throw new RuntimeException ("load: "+name+" - not a file path");
+	File zfile = ((FilePath) path).toFile();
 	if (!zfile.exists ())
 	  throw new RuntimeException ("load: "+name+" - not found");
 	if (!zfile.canRead ())
@@ -135,14 +138,13 @@ public class load extends Procedure1 {
     return apply2 (arg1, Environment.getCurrent ());
   }
 
-  public final Object apply2 (Object arg1, Object arg2)
+  public final Object apply2 (Object name, Object arg2)
     throws Throwable
   {
-    String name = arg1.toString();
     try
       {
 	Environment env = (Environment) arg2;
-	apply (name, env, relative, 0);
+	apply(Path.valueOf(name), env, relative, 0);
 	return Values.empty;
       }
     catch (java.io.FileNotFoundException e)
@@ -156,32 +158,17 @@ public class load extends Procedure1 {
       }
   }
 
-  public static final void apply (String name, Environment env,
+  public static final void apply (Path path, Environment env,
 				  boolean relative, int skipLines)
     throws Throwable
   {
-    CallContext ctx = CallContext.getInstance();
-    boolean isUri = InPort.uriSchemeSpecified(name);
-    // Resolve a relative URI.  However, if the base uri matches the
-    // default base uri (i.e. the current directory) just leave it as
-    // a filename, rather than coercing it to a URI.
-    String resolved = name;
-    if (! isUri)
-      {
-        String baseUri = ctx.getBaseUri();
-        Object base = baseUri;
-        if (! URI_utils.isAbsolute(base))
-          base = URI_utils.resolve(base, ctx.getBaseUriDefault());
-        resolved = URI_utils.resolve(name, base).toString();
-        if (relative && ! baseUri.equals(ctx.getBaseUriDefault()))
-          name = resolved;
-      }
+    String name = path.toString();
     if (name.endsWith (".zip") || name.endsWith(".jar"))
       {
-        loadCompiled (name, env);
+        loadCompiled(path, env);
         return;
       }
-    URL url = new URL(resolved);
+    URL url = path.toURL();
     char file_separator = System.getProperty ("file.separator").charAt(0);
 
     if (name.endsWith (".class"))
@@ -193,7 +180,7 @@ public class load extends Procedure1 {
         loadClassFile (name, env);
         return;
       }
-    InputStream fs = new BufferedInputStream(URI_utils.getInputStream(url));
+    InputStream fs = new BufferedInputStream(path.openInputStream());
     fs.mark(5);
     int char0 = fs.read ();
     if (char0 == -1)
@@ -210,14 +197,14 @@ public class load extends Procedure1 {
                 if (char3 == '\004')
                   {
                     fs.close ();
-                    loadCompiled (name, env);
+                    loadCompiled(path, env);
                     return;
                   }
               }
           }
       }
     fs.reset();
-    InPort src = InPort.openFile(fs, name);
+    InPort src = InPort.openFile(fs, path);
     while (--skipLines >= 0)
       src.skipRestOfLine();
     loadSource (src, env, url);
