@@ -1,17 +1,20 @@
 package kawa.standard;
 import kawa.lang.*;
-import gnu.mapping.*;
 import gnu.expr.*;
-import gnu.kawa.util.*;
+import gnu.lists.*;
 import gnu.bytecode.*;
 
 public class module_extends extends Syntax
 {
+  public static final module_extends module_extends = new module_extends();
+  static { module_extends.setName("module-extends"); }
+
   public Expression rewriteForm (Pair form, Translator tr)
   {
-    Type base = prim_method.exp2Type(((Pair) form.cdr).car, tr);
+    Type base = tr.exp2Type((Pair) form.cdr);
     ModuleExp module = tr.getModule();
     module.setSuperType((ClassType) base);
+    module.setFlag(ModuleExp.SUPERTYPE_SPECIFIED);
     return QuoteExp.voidExp;
   }
 }
