@@ -1,6 +1,5 @@
 package kawa.standard;
 import gnu.lists.*;
-import java.lang.Error;  // To work around case-fold bug in some compilers.
 import gnu.mapping.*;
 
 /**
@@ -10,6 +9,14 @@ import gnu.mapping.*;
 
 public class vector_append extends ProcedureN
 {
+  public static final vector_append vectorAppend
+    = new vector_append("vector-append");
+
+  public vector_append(String name)
+  {
+    super(name);
+  }
+
   public Object applyN (Object[] args)
   {
     return apply$V(args);
@@ -28,7 +35,7 @@ public class vector_append extends ProcedureN
 	  {
 	    int n = LList.listLength(arg, false);
 	    if (n < 0)
-	      throw new WrongType ("vector-append", i, "list or vector");
+	      throw new WrongType (vectorAppend, i, arg, "list or vector");
 	    length += n;
 	  }
       }
