@@ -1,4 +1,4 @@
-(test-init "macros" 94)
+(test-init "macros" 96)
 
 (test 'ok 'letxx (let ((xx #f)) (cond (#t xx 'ok))))
 
@@ -455,3 +455,27 @@
 	   (begin expr "this fails")))))
     (outer "third")))
 (test "this fails" 'savannah-bug-17984 thisfails)
+
+;; Savannah bug #18504 Margus Freudenthal <margus@cyber.ee>
+;; Cannot generate (define-simple-class) using syntax-case macros
+(define-syntax aa
+  (lambda (x)
+    (syntax-case x ()
+      ((_ cl arg argtype)
+       #`(define-simple-class cl ()
+           (arg type: argtype))))))
+(aa MyClass myparam <String>)
+(define aa-instance (MyClass myparam: "sarg"))
+(test (as <String> "sarg") 'savannah-bug-18504 aa-instance:myparam)
+
+;; Savannah bug #18105: Chris Wegrzyn <chris.wegrzyn@gmail.com>
+;; with-syntax causes NullPointerException during compilation but not in repl
+(begin
+  (define-syntax crashing-syntax
+    (lambda (x)
+      (syntax-case x ()
+	((k args e1)
+	 (with-syntax ((bodye1 (syntax e1)))
+		      (syntax
+		       (lambda args (begin bodye1))))))))
+  (test 3 'savannah-bug-18105 ((crashing-syntax (arg1 arg2) 3) 1 2)))
